@@ -17,13 +17,13 @@
 # Written by:
 #
 #   Marvin N. Wright
-# Institut für Medizinische Biometrie und Statistik
-# Universität zu Lübeck
+# Institut fuer Medizinische Biometrie und Statistik
+# Universitaet zu Luebeck
 # Ratzeburger Allee 160
-# 23562 Lübeck
+# 23562 Luebeck
+# Germany
 #
 # http://www.imbs-luebeck.de
-# wright@imbs.uni-luebeck.de
 # -------------------------------------------------------------------------------
 
 ##' @export
@@ -40,7 +40,7 @@ timepoints <- function(x, ...)  UseMethod("timepoints")
 ##' @author Marvin N. Wright
 ##' @export
 timepoints.ranger.prediction <- function(x, ...) {
-  if (class(x) != "ranger.predicton") {
+  if (class(x) != "ranger.prediction") {
     stop("Object ist no ranger.prediction object.")
   }
   if (x$treetype != "Survival") {
