@@ -1,29 +1,12 @@
 /*-------------------------------------------------------------------------------
- This file is part of Ranger.
+ This file is part of ranger.
 
- Ranger is free software: you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation, either version 3 of the License, or
- (at your option) any later version.
+ Copyright (c) [2014-2018] [Marvin N. Wright]
 
- Ranger is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- GNU General Public License for more details.
+ This software may be modified and distributed under the terms of the MIT license.
 
- You should have received a copy of the GNU General Public License
- along with Ranger. If not, see <http://www.gnu.org/licenses/>.
-
- Written by:
-
- Marvin N. Wright
- Institut für Medizinische Biometrie und Statistik
- Universität zu Lübeck
- Ratzeburger Allee 160
- 23562 Lübeck
- Germany
-
- http://www.imbs-luebeck.de
+ Please note that the C++ core of ranger is distributed under MIT license and the
+ R package "ranger" under GPL3 license.
  #-------------------------------------------------------------------------------*/
 
 #ifndef TREE_H_
@@ -32,10 +15,13 @@
 #include <vector>
 #include <random>
 #include <iostream>
+#include <stdexcept>
 
 #include "globals.h"
 #include "Data.h"
 
+namespace ranger {
+
 class Tree {
 public:
   Tree();
@@ -44,14 +30,17 @@ public:
   Tree(std::vector<std::vector<size_t>>& child_nodeIDs, std::vector<size_t>& split_varIDs,
       std::vector<double>& split_values);
 
-  virtual ~Tree();
+  virtual ~Tree() = default;
 
-  void init(Data* data, uint mtry, size_t dependent_varID, size_t num_samples, uint seed,
+  Tree(const Tree&) = delete;
+  Tree& operator=(const Tree&) = delete;
+
+  void init(const Data* data, uint mtry, size_t dependent_varID, size_t num_samples, uint seed,
       std::vector<size_t>* deterministic_varIDs, std::vector<size_t>* split_select_varIDs,
       std::vector<double>* split_select_weights, ImportanceMode importance_mode, uint min_node_size,
       bool sample_with_replacement, bool memory_saving_splitting, SplitRule splitrule,
-      std::vector<double>* case_weights, bool keep_inbag, double sample_fraction, double alpha, double minprop,
-      bool holdout, uint num_random_splits);
+      std::vector<double>* case_weights, bool keep_inbag, std::vector<double>* sample_fraction, double alpha,
+      double minprop, bool holdout, uint num_random_splits);
 
   virtual void allocateMemory() = 0;
 
@@ -59,12 +48,12 @@ public:
 
   void predict(const Data* prediction_data, bool oob_prediction);
 
-  void computePermutationImportance(std::vector<double>* forest_importance, std::vector<double>* forest_variance);
+  void computePermutationImportance(std::vector<double>& forest_importance, std::vector<double>& forest_variance);
 
   void appendToFile(std::ofstream& file);
   virtual void appendToFileInternal(std::ofstream& file) = 0;
 
-  const std::vector<std::vector<size_t> >& getChildNodeIDs() const {
+  const std::vector<std::vector<size_t>>& getChildNodeIDs() const {
     return child_nodeIDs;
   }
   const std::vector<double>& getSplitValues() const {
@@ -105,6 +94,9 @@ protected:
   void bootstrapWeighted();
   void bootstrapWithoutReplacementWeighted();
 
+  virtual void bootstrapClassWise();
+  virtual void bootstrapWithoutReplacementClassWise();
+
   virtual void cleanUpInternal() = 0;
 
   size_t dependent_varID;
@@ -121,12 +113,12 @@ protected:
 
   // Weight vector for selecting possible split variables, one weight between 0 (never select) and 1 (always select) for each variable
   // Deterministic variables are always selected
-  std::vector<size_t>* deterministic_varIDs;
-  std::vector<size_t>* split_select_varIDs;
-  std::vector<double>* split_select_weights;
+  const std::vector<size_t>* deterministic_varIDs;
+  const std::vector<size_t>* split_select_varIDs;
+  const std::vector<double>* split_select_weights;
 
   // Bootstrap weights
-  std::vector<double>* case_weights;
+  const std::vector<double>* case_weights;
 
   // Splitting variable for each node
   std::vector<size_t> split_varIDs;
@@ -155,7 +147,7 @@ protected:
   std::mt19937_64 random_number_generator;
 
   // Pointer to original data
-  Data* data;
+  const Data* data;
 
   // Variable importance for all variables
   std::vector<double>* variable_importance;
@@ -166,16 +158,15 @@ protected:
   std::vector<size_t> prediction_terminal_nodeIDs;
 
   bool sample_with_replacement;
-  double sample_fraction;
+  const std::vector<double>* sample_fraction;
 
   bool memory_saving_splitting;
   SplitRule splitrule;
   double alpha;
   double minprop;
   uint num_random_splits;
-
-private:
-  DISALLOW_COPY_AND_ASSIGN(Tree);
 };
 
+} // namespace ranger
+
 #endif /* TREE_H_ */
