@@ -1,29 +1,12 @@
 /*-------------------------------------------------------------------------------
- This file is part of Ranger.
+ This file is part of ranger.
 
- Ranger is free software: you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation, either version 3 of the License, or
- (at your option) any later version.
+ Copyright (c) [2014-2018] [Marvin N. Wright]
 
- Ranger is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- GNU General Public License for more details.
+ This software may be modified and distributed under the terms of the MIT license.
 
- You should have received a copy of the GNU General Public License
- along with Ranger. If not, see <http://www.gnu.org/licenses/>.
-
- Written by:
-
- Marvin N. Wright
- Institut für Medizinische Biometrie und Statistik
- Universität zu Lübeck
- Ratzeburger Allee 160
- 23562 Lübeck
- Germany
-
- http://www.imbs-luebeck.de
+ Please note that the C++ core of ranger is distributed under MIT license and the
+ R package "ranger" under GPL3 license.
  #-------------------------------------------------------------------------------*/
 
 #include <unordered_map>
@@ -31,25 +14,24 @@
 #include <algorithm>
 #include <iostream>
 #include <iterator>
-#include <vector>
 
 #include "TreeClassification.h"
 #include "utility.h"
 #include "Data.h"
 
-TreeClassification::TreeClassification(std::vector<double>* class_values, std::vector<uint>* response_classIDs) :
-    class_values(class_values), response_classIDs(response_classIDs), counter(0), counter_per_class(0) {
+namespace ranger {
+
+TreeClassification::TreeClassification(std::vector<double>* class_values, std::vector<uint>* response_classIDs,
+    std::vector<std::vector<size_t>>* sampleIDs_per_class, std::vector<double>* class_weights) :
+    class_values(class_values), response_classIDs(response_classIDs), sampleIDs_per_class(sampleIDs_per_class), class_weights(
+        class_weights), counter(0), counter_per_class(0) {
 }
 
 TreeClassification::TreeClassification(std::vector<std::vector<size_t>>& child_nodeIDs,
     std::vector<size_t>& split_varIDs, std::vector<double>& split_values, std::vector<double>* class_values,
     std::vector<uint>* response_classIDs) :
-    Tree(child_nodeIDs, split_varIDs, split_values), class_values(class_values), response_classIDs(response_classIDs), counter(
-        0), counter_per_class(0) {
-}
-
-TreeClassification::~TreeClassification() {
-  // Empty on purpose
+    Tree(child_nodeIDs, split_varIDs, split_values), class_values(class_values), response_classIDs(response_classIDs), sampleIDs_per_class(
+        0), class_weights(0), counter { }, counter_per_class { } {
 }
 
 void TreeClassification::allocateMemory() {
@@ -63,22 +45,24 @@ void TreeClassification::allocateMemory() {
       max_num_splits = num_random_splits;
     }
 
-    counter = new size_t[max_num_splits];
-    counter_per_class = new size_t[num_classes * max_num_splits];
+    counter.resize(max_num_splits);
+    counter_per_class.resize(num_classes * max_num_splits);
   }
 }
 
 double TreeClassification::estimate(size_t nodeID) {
 
   // Count classes over samples in node and return class with maximum count
-  std::unordered_map<double, size_t> class_count;
+  std::vector<double> class_count = std::vector<double>(class_values->size(), 0.0);
+
   for (size_t i = 0; i < sampleIDs[nodeID].size(); ++i) {
-    double value = data->get(sampleIDs[nodeID][i], dependent_varID);
-    ++class_count[value];
+    size_t value = (*response_classIDs)[sampleIDs[nodeID][i]];
+    class_count[value] += (*class_weights)[value];
   }
 
   if (sampleIDs[nodeID].size() > 0) {
-    return (mostFrequentValue(class_count, random_number_generator));
+    size_t result_classID = mostFrequentClass(class_count, random_number_generator);
+    return ((*class_values)[result_classID]);
   } else {
     throw std::runtime_error("Error: Empty node.");
   }
@@ -156,7 +140,7 @@ bool TreeClassification::findBestSplit(size_t nodeID, std::vector<size_t>& possi
   size_t best_varID = 0;
   double best_value = 0;
 
-  size_t* class_counts = new size_t[num_classes]();
+  std::vector<size_t> class_counts(num_classes);
   // Compute overall class counts
   for (size_t i = 0; i < num_samples_node; ++i) {
     size_t sampleID = sampleIDs[nodeID][i];
@@ -190,8 +174,6 @@ bool TreeClassification::findBestSplit(size_t nodeID, std::vector<size_t>& possi
     }
   }
 
-  delete[] class_counts;
-
   // Stop if no good split found
   if (best_decrease < 0) {
     return true;
@@ -208,8 +190,9 @@ bool TreeClassification::findBestSplit(size_t nodeID, std::vector<size_t>& possi
   return false;
 }
 
-void TreeClassification::findBestSplitValueSmallQ(size_t nodeID, size_t varID, size_t num_classes, size_t* class_counts,
-    size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease) {
+void TreeClassification::findBestSplitValueSmallQ(size_t nodeID, size_t varID, size_t num_classes,
+    const std::vector<size_t>& class_counts, size_t num_samples_node, double& best_value, size_t& best_varID,
+    double& best_decrease) {
 
 // Create possible split values
   std::vector<double> possible_split_values;
@@ -220,20 +203,25 @@ void TreeClassification::findBestSplitValueSmallQ(size_t nodeID, size_t varID, s
     return;
   }
 
-// Initialize with 0, if not in memory efficient mode, use pre-allocated space
-// -1 because no split possible at largest value
-  size_t num_splits = possible_split_values.size() - 1;
-  size_t* class_counts_right;
-  size_t* n_right;
+  // -1 because no split possible at largest value
+  const size_t num_splits = possible_split_values.size() - 1;
   if (memory_saving_splitting) {
-    class_counts_right = new size_t[num_splits * num_classes]();
-    n_right = new size_t[num_splits]();
+    std::vector<size_t> class_counts_right(num_splits * num_classes), n_right(num_splits);
+    findBestSplitValueSmallQ(nodeID, varID, num_classes, class_counts, num_samples_node, best_value, best_varID,
+        best_decrease, possible_split_values, class_counts_right, n_right);
   } else {
-    class_counts_right = counter_per_class;
-    n_right = counter;
-    std::fill(class_counts_right, class_counts_right + num_splits * num_classes, 0);
-    std::fill(n_right, n_right + num_splits, 0);
+    std::fill_n(counter_per_class.begin(), num_splits * num_classes, 0);
+    std::fill_n(counter.begin(), num_splits, 0);
+    findBestSplitValueSmallQ(nodeID, varID, num_classes, class_counts, num_samples_node, best_value, best_varID,
+        best_decrease, possible_split_values, counter_per_class, counter);
   }
+}
+
+void TreeClassification::findBestSplitValueSmallQ(size_t nodeID, size_t varID, size_t num_classes,
+    const std::vector<size_t>& class_counts, size_t num_samples_node, double& best_value, size_t& best_varID,
+    double& best_decrease, const std::vector<double>& possible_split_values, std::vector<size_t>& class_counts_right,
+    std::vector<size_t>& n_right) {
+  const size_t num_splits = possible_split_values.size() - 1;
 
   // Count samples in right child per class and possbile split
   for (auto& sampleID : sampleIDs[nodeID]) {
@@ -267,8 +255,8 @@ void TreeClassification::findBestSplitValueSmallQ(size_t nodeID, size_t varID, s
       size_t class_count_right = class_counts_right[i * num_classes + j];
       size_t class_count_left = class_counts[j] - class_count_right;
 
-      sum_right += class_count_right * class_count_right;
-      sum_left += class_count_left * class_count_left;
+      sum_right += (*class_weights)[j] * class_count_right * class_count_right;
+      sum_left += (*class_weights)[j] * class_count_left * class_count_left;
     }
 
     // Decrease of impurity
@@ -286,20 +274,16 @@ void TreeClassification::findBestSplitValueSmallQ(size_t nodeID, size_t varID, s
       }
     }
   }
-
-  if (memory_saving_splitting) {
-    delete[] class_counts_right;
-    delete[] n_right;
-  }
 }
 
-void TreeClassification::findBestSplitValueLargeQ(size_t nodeID, size_t varID, size_t num_classes, size_t* class_counts,
-    size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease) {
+void TreeClassification::findBestSplitValueLargeQ(size_t nodeID, size_t varID, size_t num_classes,
+    const std::vector<size_t>& class_counts, size_t num_samples_node, double& best_value, size_t& best_varID,
+    double& best_decrease) {
 
   // Set counters to 0
   size_t num_unique = data->getNumUniqueDataValues(varID);
-  std::fill(counter_per_class, counter_per_class + num_unique * num_classes, 0);
-  std::fill(counter, counter + num_unique, 0);
+  std::fill_n(counter_per_class.begin(), num_unique * num_classes, 0);
+  std::fill_n(counter.begin(), num_unique, 0);
 
   // Count values
   for (auto& sampleID : sampleIDs[nodeID]) {
@@ -311,7 +295,7 @@ void TreeClassification::findBestSplitValueLargeQ(size_t nodeID, size_t varID, s
   }
 
   size_t n_left = 0;
-  size_t* class_counts_left = new size_t[num_classes]();
+  std::vector<size_t> class_counts_left(num_classes);
 
   // Compute decrease of impurity for each split
   for (size_t i = 0; i < num_unique - 1; ++i) {
@@ -336,8 +320,8 @@ void TreeClassification::findBestSplitValueLargeQ(size_t nodeID, size_t varID, s
       class_counts_left[j] += counter_per_class[i * num_classes + j];
       size_t class_count_right = class_counts[j] - class_counts_left[j];
 
-      sum_left += class_counts_left[j] * class_counts_left[j];
-      sum_right += class_count_right * class_count_right;
+      sum_left += (*class_weights)[j] * class_counts_left[j] * class_counts_left[j];
+      sum_right += (*class_weights)[j] * class_count_right * class_count_right;
     }
 
     // Decrease of impurity
@@ -362,12 +346,11 @@ void TreeClassification::findBestSplitValueLargeQ(size_t nodeID, size_t varID, s
       }
     }
   }
-
-  delete[] class_counts_left;
 }
 
 void TreeClassification::findBestSplitValueUnordered(size_t nodeID, size_t varID, size_t num_classes,
-    size_t* class_counts, size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease) {
+    const std::vector<size_t>& class_counts, size_t num_samples_node, double& best_value, size_t& best_varID,
+    double& best_decrease) {
 
   // Create possible split values
   std::vector<double> factor_levels;
@@ -397,7 +380,7 @@ void TreeClassification::findBestSplitValueUnordered(size_t nodeID, size_t varID
     }
 
     // Initialize
-    size_t* class_counts_right = new size_t[num_classes]();
+    std::vector<size_t> class_counts_right(num_classes);
     size_t n_right = 0;
 
     // Count classes in left and right child
@@ -422,8 +405,8 @@ void TreeClassification::findBestSplitValueUnordered(size_t nodeID, size_t varID
       size_t class_count_right = class_counts_right[j];
       size_t class_count_left = class_counts[j] - class_count_right;
 
-      sum_right += class_count_right * class_count_right;
-      sum_left += class_count_left * class_count_left;
+      sum_right += (*class_weights)[j] * class_count_right * class_count_right;
+      sum_left += (*class_weights)[j] * class_count_left * class_count_left;
     }
 
     // Decrease of impurity
@@ -435,8 +418,6 @@ void TreeClassification::findBestSplitValueUnordered(size_t nodeID, size_t varID
       best_varID = varID;
       best_decrease = decrease;
     }
-
-    delete[] class_counts_right;
   }
 }
 
@@ -448,7 +429,7 @@ bool TreeClassification::findBestSplitExtraTrees(size_t nodeID, std::vector<size
   size_t best_varID = 0;
   double best_value = 0;
 
-  size_t* class_counts = new size_t[num_classes]();
+  std::vector<size_t> class_counts(num_classes);
   // Compute overall class counts
   for (size_t i = 0; i < num_samples_node; ++i) {
     size_t sampleID = sampleIDs[nodeID][i];
@@ -468,8 +449,6 @@ bool TreeClassification::findBestSplitExtraTrees(size_t nodeID, std::vector<size
     }
   }
 
-  delete[] class_counts;
-
   // Stop if no good split found
   if (best_decrease < 0) {
     return true;
@@ -487,7 +466,8 @@ bool TreeClassification::findBestSplitExtraTrees(size_t nodeID, std::vector<size
 }
 
 void TreeClassification::findBestSplitValueExtraTrees(size_t nodeID, size_t varID, size_t num_classes,
-    size_t* class_counts, size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease) {
+    const std::vector<size_t>& class_counts, size_t num_samples_node, double& best_value, size_t& best_varID,
+    double& best_decrease) {
 
   // Get min/max values of covariate in node
   double min;
@@ -507,19 +487,24 @@ void TreeClassification::findBestSplitValueExtraTrees(size_t nodeID, size_t varI
     possible_split_values.push_back(udist(random_number_generator));
   }
 
-  // Initialize with 0, if not in memory efficient mode, use pre-allocated space
-  size_t num_splits = possible_split_values.size();
-  size_t* class_counts_right;
-  size_t* n_right;
+  const size_t num_splits = possible_split_values.size();
   if (memory_saving_splitting) {
-    class_counts_right = new size_t[num_splits * num_classes]();
-    n_right = new size_t[num_splits]();
+    std::vector<size_t> class_counts_right(num_splits * num_classes), n_right(num_splits);
+    findBestSplitValueExtraTrees(nodeID, varID, num_classes, class_counts, num_samples_node, best_value, best_varID,
+        best_decrease, possible_split_values, class_counts_right, n_right);
   } else {
-    class_counts_right = counter_per_class;
-    n_right = counter;
-    std::fill(class_counts_right, class_counts_right + num_splits * num_classes, 0);
-    std::fill(n_right, n_right + num_splits, 0);
+    std::fill_n(counter_per_class.begin(), num_splits * num_classes, 0);
+    std::fill_n(counter.begin(), num_splits, 0);
+    findBestSplitValueExtraTrees(nodeID, varID, num_classes, class_counts, num_samples_node, best_value, best_varID,
+        best_decrease, possible_split_values, counter_per_class, counter);
   }
+}
+
+void TreeClassification::findBestSplitValueExtraTrees(size_t nodeID, size_t varID, size_t num_classes,
+    const std::vector<size_t>& class_counts, size_t num_samples_node, double& best_value, size_t& best_varID,
+    double& best_decrease, const std::vector<double>& possible_split_values, std::vector<size_t>& class_counts_right,
+    std::vector<size_t>& n_right) {
+  const size_t num_splits = possible_split_values.size();
 
   // Count samples in right child per class and possbile split
   for (auto& sampleID : sampleIDs[nodeID]) {
@@ -553,8 +538,8 @@ void TreeClassification::findBestSplitValueExtraTrees(size_t nodeID, size_t varI
       size_t class_count_right = class_counts_right[i * num_classes + j];
       size_t class_count_left = class_counts[j] - class_count_right;
 
-      sum_right += class_count_right * class_count_right;
-      sum_left += class_count_left * class_count_left;
+      sum_right += (*class_weights)[j] * class_count_right * class_count_right;
+      sum_left += (*class_weights)[j] * class_count_left * class_count_left;
     }
 
     // Decrease of impurity
@@ -567,15 +552,11 @@ void TreeClassification::findBestSplitValueExtraTrees(size_t nodeID, size_t varI
       best_decrease = decrease;
     }
   }
-
-  if (memory_saving_splitting) {
-    delete[] class_counts_right;
-    delete[] n_right;
-  }
 }
 
 void TreeClassification::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t varID, size_t num_classes,
-    size_t* class_counts, size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease) {
+    const std::vector<size_t>& class_counts, size_t num_samples_node, double& best_value, size_t& best_varID,
+    double& best_decrease) {
 
   size_t num_unique_values = data->getNumUniqueDataValues(varID);
 
@@ -633,7 +614,7 @@ void TreeClassification::findBestSplitValueExtraTreesUnordered(size_t nodeID, si
     }
 
     // Initialize
-    size_t* class_counts_right = new size_t[num_classes]();
+    std::vector<size_t> class_counts_right(num_classes);
     size_t n_right = 0;
 
     // Count classes in left and right child
@@ -658,8 +639,8 @@ void TreeClassification::findBestSplitValueExtraTreesUnordered(size_t nodeID, si
       size_t class_count_right = class_counts_right[j];
       size_t class_count_left = class_counts[j] - class_count_right;
 
-      sum_right += class_count_right * class_count_right;
-      sum_left += class_count_left * class_count_left;
+      sum_right += (*class_weights)[j] * class_count_right * class_count_right;
+      sum_left += (*class_weights)[j] * class_count_left * class_count_left;
     }
 
     // Decrease of impurity
@@ -671,8 +652,6 @@ void TreeClassification::findBestSplitValueExtraTreesUnordered(size_t nodeID, si
       best_varID = varID;
       best_decrease = decrease;
     }
-
-    delete[] class_counts_right;
   }
 }
 
@@ -706,3 +685,67 @@ void TreeClassification::addGiniImportance(size_t nodeID, size_t varID, double d
     (*variable_importance)[tempvarID] += best_gini;
   }
 }
+
+void TreeClassification::bootstrapClassWise() {
+  // Number of samples is sum of sample fraction * number of samples
+  size_t num_samples_inbag = 0;
+  double sum_sample_fraction = 0;
+  for (auto& s : *sample_fraction) {
+    num_samples_inbag += (size_t) num_samples * s;
+    sum_sample_fraction += s;
+  }
+
+  // Reserve space, reserve a little more to be save)
+  sampleIDs[0].reserve(num_samples_inbag);
+  oob_sampleIDs.reserve(num_samples * (exp(-sum_sample_fraction) + 0.1));
+
+  // Start with all samples OOB
+  inbag_counts.resize(num_samples, 0);
+
+  // Draw samples for each class
+  for (size_t i = 0; i < sample_fraction->size(); ++i) {
+    // Draw samples of class with replacement as inbag and mark as not OOB
+    size_t num_samples_class = (*sampleIDs_per_class)[i].size();
+    size_t num_samples_inbag_class = num_samples * (*sample_fraction)[i];
+    std::uniform_int_distribution<size_t> unif_dist(0, num_samples_class - 1);
+    for (size_t s = 0; s < num_samples_inbag_class; ++s) {
+      size_t draw = (*sampleIDs_per_class)[i][unif_dist(random_number_generator)];
+      sampleIDs[0].push_back(draw);
+      ++inbag_counts[draw];
+    }
+  }
+
+  // Save OOB samples
+  for (size_t s = 0; s < inbag_counts.size(); ++s) {
+    if (inbag_counts[s] == 0) {
+      oob_sampleIDs.push_back(s);
+    }
+  }
+  num_samples_oob = oob_sampleIDs.size();
+
+  if (!keep_inbag) {
+    inbag_counts.clear();
+    inbag_counts.shrink_to_fit();
+  }
+}
+
+void TreeClassification::bootstrapWithoutReplacementClassWise() {
+  // Draw samples for each class
+  for (size_t i = 0; i < sample_fraction->size(); ++i) {
+    size_t num_samples_class = (*sampleIDs_per_class)[i].size();
+    size_t num_samples_inbag_class = num_samples * (*sample_fraction)[i];
+
+    shuffleAndSplitAppend(sampleIDs[0], oob_sampleIDs, num_samples_class, num_samples_inbag_class,
+        (*sampleIDs_per_class)[i], random_number_generator);
+  }
+
+  if (keep_inbag) {
+    // All observation are 0 or 1 times inbag
+    inbag_counts.resize(num_samples, 1);
+    for (size_t i = 0; i < oob_sampleIDs.size(); i++) {
+      inbag_counts[oob_sampleIDs[i]] = 0;
+    }
+  }
+}
+
+} // namespace ranger
