@@ -1,39 +1,19 @@
 /*-------------------------------------------------------------------------------
-This file is part of Ranger.
+This file is part of ranger.
 
-Ranger is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+Copyright (c) [2014-2018] [Marvin N. Wright]
 
-Ranger is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
+This software may be modified and distributed under the terms of the MIT license.
 
-You should have received a copy of the GNU General Public License
-along with Ranger. If not, see <http://www.gnu.org/licenses/>.
-
-Written by:
-
-Marvin N. Wright
-Institut für Medizinische Biometrie und Statistik
-Universität zu Lübeck
-Ratzeburger Allee 160
-23562 Lübeck
-Germany
-
-http://www.imbs-luebeck.de
+Please note that the C++ core of ranger is distributed under MIT license and the
+R package "ranger" under GPL3 license.
 #-------------------------------------------------------------------------------*/
 
 #ifndef GLOBALS_H_
 #define GLOBALS_H_
 
-#define DISALLOW_COPY_AND_ASSIGN(TypeName) \
-    TypeName(const TypeName&);             \
-    void operator=(const TypeName&)
+namespace ranger {
 
-// Pi
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
 #endif
@@ -112,10 +92,14 @@ const double DEFAULT_MINPROP = 0.1;
 const PredictionType DEFAULT_PREDICTIONTYPE = RESPONSE;
 const uint DEFAULT_NUM_RANDOM_SPLITS = 1;
 
+//const std::vector<double> DEFAULT_SAMPLE_FRACTION = std::vector<double>({1});
+
 // Interval to print progress in seconds
 const double STATUS_INTERVAL = 30.0;
 
 // Threshold for q value split method switch
 const double Q_THRESHOLD = 0.02;
 
+} // namespace ranger
+
 #endif /* GLOBALS_H_ */
