@@ -116,82 +116,7 @@ test_that("predict.all works for single observation", {
   expect_equal(dim(pred$predictions), c(1, rf$num.trees))
 })
 
-test_that("standard error prediction working for regression", {
-  idx <- sample(nrow(iris), 10)
-  test <- iris[idx, ]
-  train <- iris[-idx, ]
-  
-  rf <- ranger(Petal.Length ~ ., train, num.trees = 5, keep.inbag = TRUE)
-  pred <- predict(rf, test, type = "se")
-  
-  expect_equal(length(pred$predictions), nrow(test))
+test_that("Warning if predicting with corrected impurity importance", {
+  rf <- ranger(Species ~ ., iris, num.trees = 5, importance = "impurity_corrected")
+  expect_warning(predict(rf, iris))
 })
-
-test_that("standard error prediction not working for other tree types", {
-  rf <- ranger(Species ~ ., iris, num.trees = 5, keep.inbag = TRUE)
-  expect_error(predict(rf, iris, type = "se"), 
-               "Error: Standard error prediction currently only available for regression.")
-  
-  rf <- ranger(Species ~ ., iris, num.trees = 5, keep.inbag = TRUE, probability = TRUE)
-  expect_error(predict(rf, iris, type = "se"), 
-               "Error: Standard error prediction currently only available for regression.")
-  
-  rf <- ranger(Surv(time, status) ~ ., veteran, num.trees = 5, keep.inbag = TRUE)
-  expect_error(predict(rf, veteran, type = "se"), 
-               "Error: Standard error prediction currently only available for regression.")
-})
-
-test_that("standard error prediction not working if keep.inbag = FALSE", {
-  rf <- ranger(Petal.Length ~ ., iris, num.trees = 5)
-  expect_error(predict(rf, iris, type = "se"), 
-               "Error: No saved inbag counts in ranger object. Please set keep.inbag=TRUE when calling ranger.")
-})
-
-test_that("standard error prediction not working if no OOB observations", {
-  test <- iris[-1, ]
-  train <- iris[1, ]
-  rf <- ranger(Petal.Length ~ ., train, num.trees = 5, keep.inbag = TRUE)
-  expect_error(predict(rf, iris, type = "se"), 
-               "Error: No OOB observations found, consider increasing num.trees or reducing sample.fraction.")
-})
-
-test_that("standard error prediction working for single testing observation", {
-  test <- iris[1, ]
-  train <- iris[-1, ]
-  
-  rf <- ranger(Petal.Length ~ ., train, num.trees = 5, keep.inbag = TRUE)
-  pred <- predict(rf, test, type = "se")
-  
-  expect_equal(length(pred$predictions), nrow(test))
-})
-
-test_that("standard error response prediction is the same as response prediction", {
-  idx <- sample(nrow(iris), 10)
-  test <- iris[idx, ]
-  train <- iris[-idx, ]
-  
-  set.seed(100)
-  rf_se <- ranger(Petal.Length ~ ., train, num.trees = 5, keep.inbag = TRUE)
-  pred_se <- predict(rf_se, test, type = "se")
-  
-  set.seed(100)
-  rf_resp <- ranger(Petal.Length ~ ., train, num.trees = 5)
-  pred_resp <- predict(rf_resp, test, type = "response")
-  
-  expect_equal(pred_se$predictions, pred_resp$predictions)
-})
-
-test_that("standard error is larger for fewer trees", {
-  idx <- sample(nrow(iris), 10)
-  test <- iris[idx, ]
-  train <- iris[-idx, ]
-  
-  rf5 <- ranger(Petal.Length ~ ., train, num.trees = 5, keep.inbag = TRUE)
-  pred5 <- predict(rf5, test, type = "se")
-  
-  rf50 <- ranger(Petal.Length ~ ., train, num.trees = 50, keep.inbag = TRUE)
-  pred50 <- predict(rf50, test, type = "se")
-  
-  expect_lt(mean(pred50$se), mean(pred5$se))
-})
-
